@@ -1,4 +1,4 @@
-import awkward._v2 as ak
+import awkward as ak
 import numpy as np
 
 
@@ -54,7 +54,7 @@ def subset(ds: ak.Array, criteria: dict) -> ak.Array:
     mask_traj = ak.ones_like(ds[ds.fields[0]], dtype="bool")
     mask_obs = ak.ones_like(ds.obs[ds.obs.fields[0]], dtype="bool")
 
-    for key in list(criteria.keys()):
+    for key in criteria.keys():
         if key in ds.fields:
             mask_traj = np.logical_and(mask_traj, mask_var(ds[key], criteria[key]))
         elif key in ds.obs.fields:
@@ -69,9 +69,9 @@ def subset(ds: ak.Array, criteria: dict) -> ak.Array:
         print("Empty set.")
         return ak.Array([])
     else:  # apply the filtering for both dimensions
-        ds_subset = ak.packed(ds[mask_traj])
+        ds_subset = ak.to_packed(ds[mask_traj])
         ds_subset = ak.with_field(
-            ds_subset, ak.packed(ds.obs[mask_obs][mask_traj]), "obs"
+            ds_subset, ak.to_packed(ds.obs[mask_obs][mask_traj]), "obs"
         )
         ds_subset = ak.with_field(
             ds_subset, ak.Array([len(x) for x in ds_subset.obs.ids]), "rowsize"
